import warnings

from .base import BaseCracker

warnings.filterwarnings('ignore')


class CloudFlareCracker(BaseCracker):
    
    cracker_name = "cloudflare"
    cracker_version = "universal"    

    """
    cloudflare cracker
    :param href: 触发 cloudfalre 验证的首页地址
    :param user_agent: 请求流程使用 ua, 不传
    :param headers: 触发验证必须的 headers, 默认 {} 
    :param cookies: 触发验证必须的 cookies, 默认 {} 
    :param html: 是否需要返回 html
    :param alpha: 是否 alpha 模式, true 会更快
    :param timeout: 最大破解超时时间
    调用示例:
    cracker = CloudFlareCracker(
        href=href,
        user_token="xxx",
    )
    ret = cracker.crack()
    """
    
    # 必传参数
    must_check_params = ["href"]
    # 默认可选参数
    option_params = {
        "branch": "Master",
        "user_agent": None,
        "headers": {},
        "cookies": {},
        "html": False,
        "sitekey": "",
        "alpha": False,
        "action": None,
        "cdata": None,
        "timeout": 30
    }




from .base import BaseCracker


class TlsV1Cracker(BaseCracker):
    
    cracker_name = "tls"
    cracker_version = "v1"    

    """
    tls v1 cracker
    :param url: 请求的 url 地址
    :param method: 请求方法, get/post
    :param headers: 请求的请求头, 可以是字符串或对象, 不传默认的请求头为 { 'User-Agent': '随机 ua' }
    :param cookies: 示例: { "value": "_abck=xxx; bm_sz=xxx", "uri": "https://example.com" }
    :param proxy: 请求流程使用的代理, 支持 protocol: http/https/socks5, 无验证代理格式: {protocol}://{ip}:{port}, 有验证代理格式: {protocol}://{user}:{password}@{ip}:{port}
    :param data: post 请求流程的 post 表单, 可以是字符串或对象, { '1': '2', '3': '4' } / '1=2&3=4'	
    :param json: post 请求流程的 json 数据, 示例 { '1': '2', '3': '4' }
    :param timeout: 请求超时时间（秒）, 默认 15 秒
    :param http2: 是否 http2 协议, 默认 false
    :param redirect: 是否重定向, 默认 true
    :param ja3: 自定义 ja3 指纹, 不传表现为最新 chrome 的随机指纹
    调用示例:
    cracker = TlsV1Cracker(
        user_token="xxx",
        url="https://www.baidu.com",
        
        # debug=True,
        # proxy=proxy,
    )
    ret = cracker.crack()
    """
    
    # 必传参数
    must_check_params = ["url"]
    # 可选参数
    option_params = {
        "method": "get",
        "headers": None,
        "cookies": None,
        "proxy": None,
        "data": None,
        "json": None,
        "timeout": 15,
        "http2": False,
        "redirect": True,
        "ja3": None
    }



# -*- coding: UTF-8 -*-

import re
import random
import primp
import json
from .utils import parse_client_hints
from .base import BaseCracker
from .utils import create_session, parse_client_hints


class DatadomeCracker(BaseCracker):
    cracker_name = "datadome"
    cracker_version = "universal"

    """
    datadome
    :param href: 触发验证的页面地址
    调用示例:
    cracker = KasadaCtCracker(
        user_token="xxx",
        href="https://rendezvousparis.hermes.com/client/register",
        user_agent="Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
        proxy="user:pass@ip:port",
        debug=True,
    )
    ret = cracker.crack()
    """

    # 必传参数
    must_check_params = ["href", "proxy"]
    # 默认可选参数
    option_params = {
        "branch": "Master",
        "captcha": None,
        "captcha_url": None,
        "captcha_html": None,
        "js_url": None,
        "js_key": None,
        "did": None,
        "user_agent": None,
        "interstitial": False,
        "accept_language": "en-US,en;q=0.9",
        "country": None,
        "ip": None,
        "timezone": None,
        "geolocation": None,
        "html": False,
        "timeout": 30
    }

    def request(self):
        country = self.wanda_args.get("country")
        _ip = self.wanda_args.get("ip")
        timezone = self.wanda_args.get("timezone")
        geolocation = self.wanda_args.get("geolocation")
        
        origin = "/".join(self.href.split("/")[0:3])
        if not self.interstitial and not self.js_key and not self.captcha and not self.captcha_url and not self.captcha_html:
            if not self.user_agent:
                version = random.randint(115, 134)
                if random.random() > .4:
                    if random.random() > .5:
                        self.user_agent = f"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36"
                    else:
                        self.user_agent = f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36'
                else:
                    if random.random() > .5:
                        self.user_agent = f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0'
                    else:
                        self.user_agent = f"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0"
            else:
                version = int(re.search(r'Chrome\/(\d+)\.\d+\.\d+\.\d+', self.user_agent)[1])
                
            if 'Edg' in self.user_agent:
                impersonate_brand = "edge"
                impersonate_versions = [122, 127, 131]
            else:
                impersonate_brand = "chrome"
                impersonate_versions = [114, 116, 117, 118, 119, 120, 123, 124, 126, 127, 128, 129, 130, 131]
            
            min_version = None
            impersonate_version = None
            for v in impersonate_versions:
                cv = abs(v - version)
                if not min_version or cv < min_version:
                    min_version = cv
                    impersonate_version = v
                    
            impersonate = f'{impersonate_brand}_{impersonate_version}'
            
            proxy = None
            if self.proxy:
                proxy = "http://" + self.proxy
            self.session = primp.Client(impersonate=impersonate, impersonate_os="macos" if 'Mac' in self.user_agent else "windows", proxy=proxy)
                
            if self.cookies:
                self.session.set_cookies(origin + "/", self.cookies)

            # 跟 ua 版本对应
            sec_ch_ua = parse_client_hints(self.user_agent)
            sec_ch_ua_ptf = '"macOS"' if 'Mac' in self.user_agent else '"Windows"'

            headers = {
                'sec-ch-ua': sec_ch_ua,
                'sec-ch-ua-mobile': '?0',
                'sec-ch-ua-platform': sec_ch_ua_ptf,
                'upgrade-insecure-requests': '1',
                'user-agent': self.user_agent,                
                'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
                'sec-fetch-site': "none",
                'sec-fetch-mode': 'navigate',
                'sec-fetch-user': '?1',
                'sec-fetch-dest': "document",
                'accept-encoding': 'gzip, deflate, br, zstd',
                'accept-language': self.accept_language,
                "priority": "1"
            }
            
            response = self.session.get(self.href, headers=headers)
            if response.status_code == 403:
                dd_match = re.search(r'var dd=(\{.*?\})', response.text)
                if dd_match:
                    self.wanda_args = {
                        "href": self.href,
                        "captcha_html": response.text,
                        "user_agent": self.user_agent,
                        "cookies": {
                            "datadome": response.cookies.get("datadome") or "",
                        },
                        "proxy": self.proxy,
                        
                        "did": self.did,
                        "html": self.html,
                        
                        "branch": self.branch,
                        "is_auth": self.wanda_args["is_auth"],
                    }
                else:
                    raise Warning("代理异常或触发未知验证")
            else:
                if not self.js_key:
                    dd_js_key = re.search(r"ddjskey = .(.{30}).", response.text)
                    if dd_js_key:
                        self.js_key = dd_js_key[1]
                
                if self.js_url and self.js_key:
                    self.wanda_args = {
                        "href": self.href,
                        "js_url": self.js_url,
                        "js_key": self.js_key,
                        "user_agent": self.user_agent,
                        "cookies": {
                            "datadome": self.session.cookies.get("datadome") or "",
                        },
                        "proxy": self.proxy,
                        
                        "did": self.did,
                        "html": self.html,
                        
                        "branch": self.branch,
                        "is_auth": self.wanda_args["is_auth"],
                    }
                else:
                    self.wanda_args = {
                        "href": self.href,
                        "interstitial": True,
                        "user_agent": self.user_agent,
                        "proxy": self.proxy,
                        
                        "did": self.did,
                        "html": self.html,
                        
                        "branch": self.branch,
                        "is_auth": self.wanda_args["is_auth"],
                    }

        if country:
            self.wanda_args["country"] = country
            
        if _ip:
            self.wanda_args["ip"] = _ip
        
        if timezone:
            self.wanda_args["timezone"] = timezone
        
        if geolocation:
            self.wanda_args["geolocation"] = geolocation
    

def crack_datadome(user_token, href, proxy, js_url=None, js_key=None, parse_index=None, user_agent=None, region="hk", verifiers=[], max_retry_times=2, internal_host=True, debug=True, show_ad=False, developer_id=None):    
    language = {
        "jp": "ja-JP,ja;q=0.9",
        "us": "en-US,en;q=0.9",
        "tw": "zh-TW,zh;q=0.9",
        "hk": "zh-HK,zh;q=0.9",
        "it": "it-IT,it;q=0.9",
        "de": "de-DE,de;q=0.9",
        "es": "es-ES,es;q=0.9",
        "in": "en-IN,en;q=0.9",
        "cn": "zh-CN,zh;q=0.9",
        "fr": "fr-FR,fr;q=0.9",
        "tr": "tr-TR,tr;q=0.9",
        "ru": "ru-RU,uk;q=0.9",
        "gb": "en-GB,en;q=0.9",
        "ua": "uk-UA,uk;q=0.9",
        "ca": "en-US,en;q=0.9",
        "au": "en-AU,en;q=0.9"
    }.get(region, "zh-CN,zh;q=0.9")

    if not user_agent:
        version = random.randint(128, 136)
        user_agent = random.choice([
            f"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36",
            f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36',  
            f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0',
            f"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0",
        ])
    
    sec_ch_ua_ptf = '"macOS"' if 'Mac' in user_agent else '"Windows"'
    sec_ch_ua = parse_client_hints(user_agent)
    
    extra = {
        'sec-ch-ua': sec_ch_ua,
        'sec-ch-ua-platform': sec_ch_ua_ptf,
        'accept-language': language, 
        'user-agent': user_agent, 
    }
    
    session = create_session(user_agent, proxy)

    try:
        resp = session.get("https://ipinfo.io/json", headers={
            "user-agent": f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
        }, timeout=5).json()
    except:
        return session, None

    current_ip = resp.get("ip")
    timezone = resp.get("timezone")
    loc = resp.get("loc")
    
    origin = "/".join(href.split("/")[0:3])
    
    headers = {
        'sec-ch-ua': sec_ch_ua, 
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
        'upgrade-insecure-requests': '1', 
        'sec-ch-ua-mobile': '?0', 
        'user-agent': user_agent, 
        'sec-ch-ua-platform': sec_ch_ua_ptf,
        'Sec-Fetch-Site': 'none', 
        'Sec-Fetch-Mode': 'navigate', 
        'Sec-Fetch-User': '?1',
        'Sec-Fetch-Dest': 'document',
        'accept-encoding': 'gzip, deflate, br, zstd',
        'accept-language': language, 
        'priority': 'u=0, i'
    }

    resp = session.get(href, headers=headers)
    
    retry_times = 0
    while resp.status_code in [403, 405] and retry_times < max_retry_times:
        try:
            dd_match = re.search(r'var dd=(\{.*?\})', resp.text)
            captcha = json.loads(dd_match[1].replace("'", '"'))
            if captcha.get("t") == "bv":
                raise Warning("代理被封锁, 请切换代理重试")
        except:
            raise Warning(f"验证异常: {resp.status_code} {resp.text}")
        datadome = resp.cookies.get('datadome')
        cracker = DatadomeCracker(
            show_ad=show_ad,
            internal_host=internal_host,
            user_token=user_token,
            developer_id=developer_id,
            href=href,
            user_agent=user_agent,
            captcha_html=resp.text,
            cookies={
                'datadome': datadome
            },
            proxy=proxy,
            country=region,
            ip=current_ip,
            timezone=timezone,
            geolocation=loc,
            debug=debug
        )
        res = cracker.crack()
        if res:
            extra = cracker.extra()
            cookies = session.get_cookies(origin + "/")
            cookies.update(res)
            session = create_session(user_agent, proxy)
            session.set_cookies(origin + "/", cookies)
            if parse_index:
                resp = session.get(href, headers=headers)
                retry_times += 1
            else:
                break            
        else:
            raise Warning("验证失败")
        
    if retry_times == 0 and js_url and js_key:
        datadome = session.get_cookies(origin + "/").get('datadome')
        cookies = None
        if datadome:
            cookies = {
                "datadome": datadome
            }
        cracker = DatadomeCracker(
            show_ad=show_ad,
            internal_host=internal_host,
            user_token=user_token,
            href=href,
            js_url=js_url,
            js_key=js_key,
            user_agent=user_agent,
            cookies=cookies,
            proxy=proxy,
            country=region,
            ip=current_ip,
            timezone=timezone,
            geolocation=loc,
            debug=debug
        )
        res = cracker.crack()
        if res:
            extra = cracker.extra()
            cookies = session.get_cookies(origin + "/")
            cookies.update(res)
            session = create_session(user_agent, proxy)
            session.set_cookies(origin + "/", cookies)
        else:
            raise Warning("验证失败")

    global_args = {}
    
    if parse_index:
        parse_index(resp, global_args)
    
    for verifier in verifiers:
        
        retry_times = 0
        
        resp = verifier(session, extra, global_args)

        while resp.status_code in [403, 405] and retry_times < max_retry_times:
            did = extra.get("did")
            
            datadome = session.get_cookies(origin + "/")["datadome"]
            
            captcha_args = {}
            try:
                captcha_url = resp.json()["url"]
                if 't=bv' in captcha_url:
                    raise Warning("代理被封锁, 请切换代理重试")
                captcha_args["captcha_url"] = captcha_url
            except:
                try:
                    captcha_html = resp.text
                    dd_match = re.search(r'var dd=(\{.*?\})', resp.text)
                    captcha = json.loads(dd_match[1].replace("'", '"'))
                    if captcha.get("t") == "bv":
                        raise Warning("代理被封锁, 请切换代理重试")
                    
                    captcha_args["captcha_html"] = captcha_html
                except:
                    raise Warning("验证异常: " + resp.text)
            
            res = DatadomeCracker(
                show_ad=show_ad,
                internal_host=internal_host,
                user_token=user_token,
                href=href,
                user_agent=user_agent,
                cookies={
                    "datadome": datadome
                },
                **captcha_args,
                did=did,
                proxy=proxy,
                debug=debug
            ).crack()
            if res:
                cookies = session.get_cookies(origin + "/")
                cookies.update(res)
                session = create_session(user_agent, proxy)
                session.set_cookies(origin + "/", cookies)
                resp = verifier(session, extra, global_args)
            else:
                raise Warning("验证失败")
            
            retry_times += 1

    return session, resp





from .crackers.akamai import AkamaiV2Cracker, crack_akamai_v3
from .crackers.aws import AwsUniversalCracker
from .crackers.cloudflare import CloudFlareCracker
from .crackers.datadome import DatadomeCracker, crack_datadome
from .crackers.discord import DiscordCracker
from .crackers.hcaptcha import HcaptchaCracker
from .crackers.incapsula import (IncapsulaRbzidCracker,
                                 IncapsulaReese84Cracker,
                                 IncapsulaUtmvcCracker)
from .crackers.kasada import KasadaCdCracker, KasadaCtCracker
from .crackers.perimeterx import PerimeterxCracker
from .crackers.recaptcha import (ReCaptchaAppCracker,
                                 ReCaptchaEnterpriseCracker,
                                 ReCaptchaSteamCracker,
                                 ReCaptchaUniversalCracker)
from .crackers.shape import ShapeV1Cracker, ShapeV2Cracker
from .crackers.tls import TlsV1Cracker
from .crackers.utils import parse_client_hints, create_session

__all__ = [
    'pynocaptcha', 
    'CloudFlareCracker', 'IncapsulaReese84Cracker', 'IncapsulaUtmvcCracker', 'IncapsulaRbzidCracker', 'HcaptchaCracker', 
    'AkamaiV2Cracker', 'ReCaptchaUniversalCracker', 'ReCaptchaEnterpriseCracker', 'ReCaptchaSteamCracker',
    'TlsV1Cracker', 'DiscordCracker', 'ReCaptchaAppCracker', 'AwsUniversalCracker', 'PerimeterxCracker',
    'KasadaCtCracker', 'KasadaCdCracker', 'DatadomeCracker', 'ShapeV1Cracker', 'ShapeV2Cracker',
    'parse_client_hints', 'create_session', 'crack_akamai_v3', 'crack_datadome'
]




from .base import BaseCracker

import re
import random
import requests
from loguru import logger
from .utils import create_session, parse_client_hints


class AkamaiV2Cracker(BaseCracker):
    
    cracker_name = "akamai"
    cracker_version = "v2"    

    """
    akamai v2 cracker
    :param href: 触发验证的页面地址
    :param api: akamai 提交 sensor_data 的地址
    :param telemetry: 是否 headers 中的 telemetry 参数验证形式, 默认 false
    :param cookies: 请求 href 首页返回的 cookie _abck, bm_sz 值, 传了 api 参数必须传该值, 示例: { "value": "_abck=xxx; bm_sz=xxx", "uri": "https://example.com" }
    :param device: 请求流程使用的设备类型, 可选 pc/mobile, 默认 mobile
    调用示例:
    cracker = AkamaiV2Cracker(
        user_token="xxx",
        href="xxx",
        api="xxx",
        
        # debug=True,
        # proxy=proxy,
    )
    ret = cracker.crack()
    """
    
    # 必传参数
    must_check_params = ["href"]
    # 默认可选参数
    option_params = {
        "branch": "Master",
        "api": "",
        "telemetry": False,
        "uncheck": False,
        "sec_cpt_provider": None,
        "sec_cpt_script": None,
        "sec_cpt_key": None,
        "sec_cpt_challenge": {},
        "sec_cpt_host": None,
        "sec_cpt_html": None,
        "sec_cpt_duration": None,
        "sec_cpt_src": None,
        "sec_cpt_html": None,
        "proxy": None,
        "cookies": {},
        "country": None,
        "ip": None,
        "timezone": None,
        "geolocation": None,
        "user_agent": None,
        "timeout": 30
    }

    
def crack_akamai_v3(user_token, requests_args, proxy, other_requests=None, user_agent=None, region="hk", developer_id=None, internal_host=True, debug=False):
    try:
        language = {
            "jp": "ja-JP,ja;q=0.9",
            "us": "en-US,en;q=0.9",
            "tw": "zh-TW,zh;q=0.9",
            "hk": "zh-HK,zh;q=0.9",
            "it": "it-IT,it;q=0.9",
            "de": "de-DE,de;q=0.9",
            "es": "es-ES,es;q=0.9",
            "in": "en-IN,en;q=0.9",
            "cn": "zh-CN,zh;q=0.9",
            "fr": "fr-FR,fr;q=0.9",
            "tr": "tr-TR,tr;q=0.9",
            "ru": "ru-RU,uk;q=0.9",
            "gb": "en-GB,en;q=0.9",
            "ua": "uk-UA,uk;q=0.9",
            "ca": "en-US,en;q=0.9",
            "au": "en-AU,en;q=0.9"
        }.get(region, "zh-CN,zh;q=0.9")

        if not user_agent:
            version = random.randint(128, 136)
            user_agent = random.choice([
                f"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36",
                f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36',
                f'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0',
                f"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/{version}.0.0.0 Safari/537.36 Edg/{version}.0.0.0",
            ])
        
        sec_ch_ua_ptf = '"macOS"' if 'Mac' in user_agent else '"Windows"'
        sec_ch_ua = parse_client_hints(user_agent)
        
        session = create_session(user_agent, proxy)

        try:
            resp = session.get("https://ipinfo.io/json", headers={
                "user-agent": f'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
            }, timeout=5).json()
        except:
            raise Warning("代理超时")

        current_ip = resp.get("ip")
        timezone = resp.get("timezone")
        loc = resp.get("loc")

        href = requests_args["referer"]
        headers = {
            'sec-ch-ua': sec_ch_ua, 
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7', 
            'upgrade-insecure-requests': '1', 
            'sec-ch-ua-mobile': '?0', 
            'user-agent': user_agent, 
            'sec-ch-ua-platform': sec_ch_ua_ptf,
            'Sec-Fetch-Site': 'none', 
            'Sec-Fetch-Mode': 'navigate', 
            'Sec-Fetch-User': '?1',
            'Sec-Fetch-Dest': 'document',
            'accept-encoding': 'gzip, deflate, br, zstd',
            'Accept-Language': language, 
            'priority': 'u=0, i'
        }

        response = session.get(href, headers=headers)

        if 'Oops, Something Went Wrong.' in response.text:
            raise Warning('ip 被 ban, 切 ip 重试')

        origin = "/".join(href.split("/")[0:3])
        nocaptcha_host = "api.nocaptcha.cn" if internal_host else "api.nocaptcha.io"
        if "var chlgeId = ''" in response.text:
            if debug:
                logger.debug('触发 bm_sc 模式')
            
            bm_sc_src = re.search(r'src="(.*?)"', response.text)[1]

            resp = requests.post(
                f'http://{nocaptcha_host}/api/wanda/akamai/v2', headers={
                    'user-token': user_token,
                    **({ "Developer-Id": developer_id } if developer_id else {})
                }, json={
                    'href': href,
                    'bm_sc_src': bm_sc_src,
                    'cookies': session.get_cookies(href),
                    'user_agent': user_agent,
                    'proxy': proxy,
                    'country': region,
                    'ip': current_ip,
                    'timezone': timezone,
                    'geolocation': loc,
                }
            ).json()
            if debug:
                logger.debug(f"bm_sc 模式验证结果: {resp}")
            if resp["status"]:                
                session = create_session(user_agent, proxy)
                session.set_cookies(origin + "/", resp["data"])
                response = session.get(href, headers=headers)
            else:
                raise Warning(f'akamai bm_sc 验证失败, id: {resp["id"]}, err: {resp["msg"]}')
            
        api = requests_args.get("api")
        if not api:
            try:
                apis = re.findall("type=\"text\/javascript\"  src\=\"((?:\/[A-Za-z0-9\-\_\+]*?)+)\"\>\<\/script\>", response.text)
                if not apis:
                    apis = re.findall("nonce=\"[a-f0-9]{32}\" src\=\"((?:\/[A-Za-z0-9\-\_\+]*?)+)\"\>\<\/script\>", response.text)
                if not apis:
                    raise Warning('api 查找失败: ' + response.text)
                api = apis[0]
                if not api.startswith("http"):
                    api = origin + api
                requests_args["api"] = api
            except:
                raise Warning('api 查找失败: ' + response.text)
        
        headers = {
            'sec-ch-ua': sec_ch_ua,
            'sec-ch-ua-mobile': '?0',
            'user-agent': user_agent,
            'sec-ch-ua-platform': sec_ch_ua_ptf,
            'accept': '*/*',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'no-cors',
            'sec-fetch-dest': 'script',
            'referer': href,
            'accept-encoding': 'gzip, deflate, br, zstd',
            'accept-language': language,
            'priority': 'u=1',
        }

        api_response = session.get(api, headers=headers)
        if api_response.status_code != 200:
            raise Warning("脚本请求失败")

        if other_requests:
            other_requests(session, {
                'sec-ch-ua': sec_ch_ua,
                'user-agent': user_agent,
                'sec-ch-ua-platform': sec_ch_ua_ptf,
                'accept-language': language,
            }, requests_args)

        for k, v in requests_args.items():
            if isinstance(v, dict):
                for k1, v1 in v.items():
                    if callable(v1):
                        requests_args[k][k1] = v1(response)
        
        sub_domain = '.' + '.'.join(href.split('/')[2].split('.')[-2:])
        cookies = [
            {
                'name': name,
                'value': value,
                'domain': sub_domain
            } for name, value in session.get_cookies(origin + "/").items()
        ]
        
        resp = requests.post(
            f'http://{nocaptcha_host}/api/wanda/akamai/v3', headers={
                'user-token': user_token,
                **({ "Developer-Id": developer_id } if developer_id else {})
            }, json={
                **requests_args,
                'api_headers': dict(api_response.headers),
                'cookies': cookies,
                'user_agent': user_agent,
                'proxy': proxy,
                'country': region,
                'ip': current_ip,
                'timezone': timezone,
                'geolocation': loc,
            }
        ).json()
        if debug:
            logger.debug(f"akamai v3 验证结果: {resp}")
            
        if resp["status"]:
            cookies = resp["data"]["cookies"]
            session.set_cookies(origin + "/", cookies)
            return session, resp
        else:
            raise Warning(f'akamai v3 验证失败, id: {resp["id"]}, err: {resp["msg"]}')

    except Exception as e:
        if 'client error' in str(e):
            raise Warning('代理错误')
        else:
            raise e
